@@ -35,6 +35,7 @@
 #include <gtk/gtkclipboard.h>
 #include <gtk/gtkframe.h>
 #include <gtk/gtkhpaned.h>
+#include <gtk/gtkvpaned.h>
 #include <gtk/gtkimage.h>
 #include <gtk/gtkitemfactory.h>
 #include <gtk/gtkmain.h>
@@ -49,6 +50,7 @@
 #include <gtk/gtktreeselection.h>
 #include <gtk/gtktreeview.h>
 #include <gtk/gtkvbox.h>
+#include <gtk/gtkvpaned.h>
 #include <gtk/gtklabel.h>
 #include <libintl.h>
 #include <string.h>
@@ -141,6 +143,7 @@ gconf_editor_window_about_window (gpointer callback_data, guint action, GtkWidge
 	GtkWidget *vbox;
 	GtkWidget *label;
 	GtkWidget *image;
+	GdkPixbuf *icon;
 	
 	gchar *markup;
 	
@@ -151,6 +154,11 @@ gconf_editor_window_about_window (gpointer callback_data, guint action, GtkWidge
 					 GTK_RESPONSE_OK);
 	
 	gtk_window_set_title (GTK_WINDOW (about_window), _("About GConf-Editor"));
+	icon = gdk_pixbuf_new_from_file (IMAGEDIR"/gconf-editor.png", NULL);
+	if (icon != NULL) {
+		gtk_window_set_icon (GTK_WINDOW (about_window), icon);
+		g_object_unref (icon);
+	}
 	
 	gtk_window_set_resizable (GTK_WINDOW (about_window), FALSE);
 	gtk_window_set_position (GTK_WINDOW (about_window), 
@@ -162,7 +170,7 @@ gconf_editor_window_about_window (gpointer callback_data, guint action, GtkWidge
 	gtk_container_set_border_width (GTK_CONTAINER (vbox), 6);
 	gtk_box_pack_start (GTK_BOX (GTK_DIALOG (about_window)->vbox), vbox, FALSE, FALSE, 0);
 
-	image = gtk_image_new_from_file (GCONF_EDITOR_IMAGEDIR"/about-logo.png");
+	image = gtk_image_new_from_file (IMAGEDIR"/gconf-editor.png");
 	gtk_box_pack_start (GTK_BOX (vbox), image, FALSE, FALSE, 0);
 
 	label = gtk_label_new (NULL);
@@ -201,9 +209,26 @@ static GtkItemFactoryEntry menu_items[] =
 };
 
 static void
-gconf_editor_window_popup_error_dialog (GtkWindow *parent, const gchar *message, GError *error)
+gconf_editor_window_popup_error_dialog (GtkWindow   *parent,
+					const gchar *message,
+					GError      *error)
 {
-	g_print ("popup error dialog!\n");
+	GtkWidget *dialog;
+
+	g_return_if_fail (error != NULL);
+
+	dialog = gtk_message_dialog_new (parent,
+					 GTK_DIALOG_DESTROY_WITH_PARENT,
+					 GTK_MESSAGE_ERROR,
+					 GTK_BUTTONS_CLOSE,
+					 message,
+					 error->message);
+	g_error_free (error);
+
+	g_signal_connect (dialog, "response", G_CALLBACK (gtk_widget_destroy), NULL);
+
+	gtk_window_set_resizable (GTK_WINDOW (dialog), FALSE);
+	gtk_widget_show (dialog);
 }
 
 static void
@@ -229,13 +254,57 @@ gconf_editor_popup_window_unset_key (gpointer callback_data, guint action, GtkWi
 	}
 }
 
+static void
+gconf_editor_new_key_response (GtkDialog *editor,
+			       int        response)
+{
+	if (response == GTK_RESPONSE_OK) {
+		GConfValue *value;
+		char       *full_path;
+		char       *why_not_valid = NULL;
+		
+		full_path = gconf_key_editor_get_full_key_path (GCONF_KEY_EDITOR (editor));
+		
+		if (!gconf_valid_key (full_path, &why_not_valid)) {
+			GtkWidget *message_dialog;
+
+			g_assert (why_not_valid != NULL);
+
+			message_dialog = gtk_message_dialog_new (GTK_WINDOW (editor),
+								 GTK_DIALOG_MODAL,
+								 GTK_MESSAGE_ERROR,
+								 GTK_BUTTONS_OK,
+								 _("Could not create key. The error is:\n%s"),
+								 why_not_valid);
+			gtk_dialog_set_default_response (GTK_DIALOG (message_dialog), GTK_RESPONSE_OK);
+			gtk_dialog_run (GTK_DIALOG (message_dialog));
+			gtk_widget_destroy (message_dialog);
+
+			g_free (full_path);
+			g_free (why_not_valid);
+
+			/* leave the key editor in place */
+			return;
+		}
+			
+		/* Create the key */
+		value = gconf_key_editor_get_value (GCONF_KEY_EDITOR (editor));
+
+		gconf_client_set (gconf_client_get_default (), full_path, value, NULL);
+
+		gconf_value_free (value);
+		g_free (full_path);
+	}
+
+	gtk_widget_destroy (GTK_WIDGET (editor));
+}
+
 static void
 gconf_editor_popup_window_new_key (gpointer callback_data, guint action, GtkWidget *widget)
 {
 	GConfEditorWindow *window = GCONF_EDITOR_WINDOW (callback_data);
 	GtkTreeIter iter;
 	GtkWidget *editor;
-	int response;
 	char *path;
 	
 	editor = gconf_key_editor_new (GCONF_KEY_EDITOR_NEW_KEY);
@@ -251,48 +320,43 @@ gconf_editor_popup_window_new_key (gpointer callback_data, guint action, GtkWidg
 	}
 	
 	gconf_key_editor_set_key_path (GCONF_KEY_EDITOR (editor), path);
-	response = gtk_dialog_run (GTK_DIALOG (editor));
+	g_free (path);
 
+	g_signal_connect (editor, "response",
+			  G_CALLBACK (gconf_editor_new_key_response), NULL);
+
+	gtk_widget_show (editor);
+}
+
+static void
+gconf_editor_edit_key_response (GtkDialog *editor,
+				int        response)
+{
 	if (response == GTK_RESPONSE_OK) {
-		char *full_path, *why_not_valid;
-		
-		full_path = gconf_key_editor_get_full_key_path (GCONF_KEY_EDITOR (editor));
-		
-		if (!gconf_valid_key (full_path, &why_not_valid)) {
-			GtkWidget *message_dialog;
+		GConfValue *value;
+		GError     *error = NULL;
+		const char *path;
 
-			message_dialog = gtk_message_dialog_new (GTK_WINDOW (editor),
-								 GTK_DIALOG_MODAL,
-								 GTK_MESSAGE_ERROR,
-								 GTK_BUTTONS_OK,
-								 _("Could not create key. The error is:\n%s"),
-								 why_not_valid);
-			gtk_dialog_set_default_response (GTK_DIALOG (message_dialog), GTK_RESPONSE_OK);
-			gtk_dialog_run (GTK_DIALOG (message_dialog));
-			gtk_widget_destroy (message_dialog);
-			g_free (why_not_valid);
-		}
-		else {
-			GConfValue *value;
-			GError *error = NULL;
-			
-			/* Create the key */
-			value = gconf_key_editor_get_value (GCONF_KEY_EDITOR (editor));
+		value = gconf_key_editor_get_value (GCONF_KEY_EDITOR (editor));
 
-			gconf_client_set (gconf_client_get_default (), full_path, value, &error);
-			gconf_value_free (value);
+		path = gconf_key_editor_get_key_name (GCONF_KEY_EDITOR (editor));
+		g_assert (gconf_valid_key (path, NULL));
 
-			if (error != NULL) {
-				g_error_free (error);
-			}
-			else {
-				gtk_widget_destroy (editor);
-			}
+		gconf_client_set (gconf_client_get_default (),
+				  path, value, &error);
+
+		gconf_value_free (value);
+
+		if (error != NULL) {
+			gconf_editor_window_popup_error_dialog (GTK_WINDOW (editor),
+								_("Could not change key value. Error message:\n%s"),
+								error);
+			g_error_free (error);
+			return;
 		}
 	}
-	else {
-		gtk_widget_destroy (editor);
-	}
+
+	gtk_widget_destroy (GTK_WIDGET (editor));
 }
 
 static void
@@ -302,9 +366,7 @@ gconf_editor_popup_window_edit_key (gpointer callback_data, guint action, GtkWid
 	GtkTreeIter iter;
 	GtkWidget *editor, *dialog;
 	GConfValue *value; 
-	int response;
-	char *path;
-	GError *error = NULL;
+	char *path = NULL;
 	
 	gtk_tree_selection_get_selected (gtk_tree_view_get_selection (GTK_TREE_VIEW (window->list_view)),
 					 NULL, &iter);
@@ -324,35 +386,23 @@ gconf_editor_popup_window_edit_key (gpointer callback_data, guint action, GtkWid
 						   "version."));
 		gtk_dialog_run (GTK_DIALOG (dialog));
 		gtk_widget_destroy (dialog);
+		gconf_value_free (value);
+		g_free (path);
 		return;
 	}
-
 	
 	editor = gconf_key_editor_new (GCONF_KEY_EDITOR_EDIT_KEY);
-	gconf_key_editor_set_key_name (GCONF_KEY_EDITOR (editor), path);
+
 	gconf_key_editor_set_value (GCONF_KEY_EDITOR (editor), value);
+	gconf_value_free (value);
 
-	response = gtk_dialog_run (GTK_DIALOG (editor));
+	gconf_key_editor_set_key_name (GCONF_KEY_EDITOR (editor), path);
+	g_free (path);
 
-	if (response == GTK_RESPONSE_OK) {
-		value = gconf_key_editor_get_value (GCONF_KEY_EDITOR (editor));
+	g_signal_connect (editor, "response",
+			  G_CALLBACK (gconf_editor_edit_key_response), NULL);
 
-		gconf_client_set (gconf_client_get_default (),
-				  path, value, &error);
-
-		if (error != NULL) {
-			gconf_editor_window_popup_error_dialog (GTK_WINDOW (editor),
-								_("Could not change key value. Error message:\n%s"),
-								error);
-			g_error_free (error);
-		}
-		else {
-			gtk_widget_destroy (editor);
-		}
-	}
-	else {
-		gtk_widget_destroy (editor);
-	}
+	gtk_widget_show (editor);
 }
 
 static GtkItemFactoryEntry popup_menu_items[] = {
@@ -579,14 +629,14 @@ gconf_editor_window_class_init (GConfEditorWindowClass *klass)
 static void
 gconf_editor_window_init (GConfEditorWindow *window)
 {
-	GtkWidget *paned, *scrolled_window, *vbox;
+	GtkWidget *hpaned, *vpaned, *scrolled_window, *vbox;
 	GdkPixbuf *icon;
 	GtkTreeViewColumn *column;
 	GtkCellRenderer *cell;
 	GtkAccelGroup *accel_group;      
 	GtkItemFactory *item_factory;
 	GtkTreePath *path;
-	GtkWidget *details_frame, *pane_box, *table, *label, *text_view;
+	GtkWidget *details_frame, *table, *label, *text_view;
 	
 	/* Create popup menu */
 	window->popup_factory = gtk_item_factory_new (GTK_TYPE_MENU, "<main>", NULL);
@@ -628,9 +678,9 @@ gconf_editor_window_init (GConfEditorWindow *window)
 	gconf_bookmarks_hook_up_menu (window, gtk_item_factory_get_widget (item_factory, "/Bookmarks"));
 
 	/* Create content area */
-	paned = gtk_hpaned_new ();
-	gtk_paned_set_position (GTK_PANED (paned), 200);
-	gtk_box_pack_start (GTK_BOX (vbox), paned, TRUE, TRUE, 0);
+	hpaned = gtk_hpaned_new ();
+	gtk_paned_set_position (GTK_PANED (hpaned), 200);
+	gtk_box_pack_start (GTK_BOX (vbox), hpaned, TRUE, TRUE, 0);
 
 	/* Create status bar */
 	window->statusbar = gtk_statusbar_new ();
@@ -674,7 +724,7 @@ gconf_editor_window_init (GConfEditorWindow *window)
 					GTK_POLICY_AUTOMATIC,
 					GTK_POLICY_AUTOMATIC);
 	gtk_container_add (GTK_CONTAINER (scrolled_window), window->tree_view);
-	gtk_paned_add1 (GTK_PANED (paned), scrolled_window);
+	gtk_paned_add1 (GTK_PANED (hpaned), scrolled_window);
 
 	/* Create list model and list view */
 	window->list_model = gconf_list_model_new ();
@@ -732,13 +782,14 @@ gconf_editor_window_init (GConfEditorWindow *window)
 					GTK_POLICY_AUTOMATIC);
 	gtk_container_add (GTK_CONTAINER (scrolled_window), window->list_view);
 
-	pane_box = gtk_vbox_new (FALSE, 0);
-	gtk_box_pack_start (GTK_BOX (pane_box), scrolled_window, TRUE, TRUE, 0);
-	gtk_paned_add2 (GTK_PANED (paned), pane_box);
+	vpaned = gtk_vpaned_new ();
+	gtk_paned_add2 (GTK_PANED (hpaned), vpaned);
+
+	gtk_paned_pack1 (GTK_PANED (vpaned), scrolled_window, TRUE, FALSE);
 
 	/* Create details area */
 	details_frame = gtk_frame_new (_("Key Documentation"));
-	gtk_box_pack_start (GTK_BOX (pane_box), details_frame, FALSE, FALSE, 0);
+	gtk_paned_pack2 (GTK_PANED (vpaned), details_frame, FALSE, FALSE);
 
 	table = gtk_table_new (2, 4, FALSE);
 	gtk_table_set_row_spacings (GTK_TABLE (table), 4);
