@@ -63,6 +63,8 @@ gconf_list_model_notify_func (GConfClient* client, guint cnxn_id, GConfEntry *en
 		iter.stamp = list_model->stamp;
 		iter.user_data = list;
 
+		list_model->length++;
+
 		path = gtk_tree_model_get_path (GTK_TREE_MODEL (list_model), &iter);
 		gtk_tree_model_row_inserted (GTK_TREE_MODEL (list_model), path, &iter);
 		gtk_tree_path_free (path);
@@ -86,6 +88,7 @@ gconf_list_model_notify_func (GConfClient* client, guint cnxn_id, GConfEntry *en
 		else {
 			gtk_tree_model_row_deleted (GTK_TREE_MODEL (list_model), path);
 			list_model->values = g_slist_remove (list_model->values, list->data);
+			list_model->length--;
 			g_hash_table_remove (list_model->key_hash, key);
 		}
 	}
